==> vdr-plugin-fritzbox_0.0.3.orig.tar.gz: fritzbox-0.0.3/fritzfonbuch.c <==
#include "fritzfonbuch.h"
#include "tcpclient.h"
#include "setup.h"
#include <vdr/tools.h>
#include <sstream>

const char *Entities[97][2] = {
	{"&nbsp;",  " "},
	{"&iexcl;", ""},
	{"&cent;",  ""},
	{"&pound;", ""},
	{"&curren;",""},
	{"&yen;",   ""},
	{"&brvbar;",""},
	{"&sect;",  ""},
	{"&uml;",   ""},
	{"&copy;",  ""},
	{"&ordf;",  ""},
	{"&laquo;", ""},
	{"&not;",   ""},
	{"&shy;",   ""},
	{"&reg;",   ""},
	{"&macr;",  ""},
	{"&deg;",   ""},
	{"&plusmn;",""},
	{"&sup2;",  ""},
	{"&sup3;",  ""},
	{"&acute;", ""},
	{"&micro;", ""},
	{"&para;",  ""},
	{"&middot;",""},
	{"&cedil;", ""},
	{"&sup1;",  ""},
	{"&ordm;",  ""},
	{"&raquo;", ""},
	{"&frac14;",""},
	{"&frac12;",""},
	{"&frac34;",""},
	{"&iquest;",""},
	{"&Agrave;",""},
	{"&Aacute;",""},
	{"&Acirc;", ""},
	{"&Atilde;",""},
	{"&Auml;",  ""},
	{"&Aring;", ""},
	{"&AElig;", ""},
	{"&Ccedil;",""},
	{"&Egrave;",""},
	{"&Eacute;",""},
	{"&Ecirc;", ""},
	{"&Euml;",  ""},
	{"&Igrave;",""},
	{"&Iacute;",""},
	{"&Icirc;", ""},
	{"&Iuml;",  ""},
	{"&ETH;",   ""},
	{"&Ntilde;",""},
	{"&Ograve;",""},
	{"&Oacute;",""},
	{"&Ocirc;", ""},
	{"&Otilde;",""},
	{"&Ouml;",  ""},
	{"&times;", ""},
	{"&Oslash;",""},
	{"&Ugrave;",""},
	{"&Uacute;",""},
	{"&Ucirc;", ""},
	{"&Uuml;",  ""},
	{"&Yacute;",""},
	{"&THORN;", ""},
	{"&szlig;", ""},
	{"&agrave;",""},
	{"&aacute;",""},
	{"&acirc;", ""},
	{"&atilde;",""},
	{"&auml;",  ""},
	{"&aring;", ""},
	{"&aelig;", ""},
	{"&ccedil;",""},
	{"&egrave;",""},
	{"&eacute;",""},
	{"&ecirc;", ""},
	{"&euml;",  ""},
	{"&igrave;",""},
	{"&iacute;",""},
	{"&icirc;", ""},
	{"&iuml;",  ""},
	{"&eth;",   ""},
	{"&ntilde;",""},
	{"&ograve;",""},
	{"&oacute;",""},
	{"&ocirc;", ""},
	{"&otilde;",""},
	{"&ouml;",  ""},
	{"&divide;",""},
	{"&oslash;",""},
	{"&ugrave;",""},
	{"&uacute;",""},
	{"&ucirc;", ""},
	{"&uuml;",  ""},
	{"&yacute;",""},
	{"&thorn;", ""},
	{"&yuml;",  ""},
	{"&amp;",   "&"},
};

std::string &convertEntities(std::string &s) {
	if (s.find("&") != std::string::npos) {
		for (int i=0; i<97; i++) {
			std::string::size_type pos = s.find(Entities[i][0]);
			if (pos != std::string::npos) {
				s.replace(pos, strlen(Entities[i][0]), Entities[i][1]);
				i--; //search for the same entity again
			}
		}
	}
	return s;
}


cFritzFonbuch::cFritzFonbuch() 
:cThread("cFritzFonbuch")
{
	fonbuchRead = false;
	Start();
}

cFritzFonbuch::~cFritzFonbuch() {
}

void cFritzFonbuch::Login() {
	std::ostringstream sMsg;

	dsyslog("fritzfonbuch.c: sending login request to fritz.box.");
	cTcpClient *tc = new cTcpClient(fritzboxConfig.url, PORT_WWW);
	*tc  <<		"POST /cgi-bin/webcm HTTP/1.1\n"
		<<  	"Content-Type: application/x-www-form-urlencoded\n"
		<<  	"Content-Length: "
		<<  	23 + fritzboxConfig.password.size()
		<<  	"\n\nlogin:command/password="
		<<  	fritzboxConfig.password
		<<  	"\n";
	do {
		*tc >> sMsg;
		usleep(100000);
	} while (sMsg.str().find(std::string("</html>")) == std::string::npos);
	delete tc;
}

void cFritzFonbuch::Action() {
	std::string msg;
	do {
		try {
			fonbuchList.clear();
			Login();

			dsyslog("fritzfonbuch.c: sending fonbuch request.");
			cTcpClient tc(fritzboxConfig.url, PORT_WWW);
			tc  <<	"GET /cgi-bin/webcm?getpage=../html/de/menus/menu2.html"
					"&var:lang=de&var:pagename=fonbuch&var:menu=fon HTTP/1.1\n\n\0";

			do {
				tc >> msg;
				usleep(100000);
			} while (msg.find(std::string("</html>")) == std::string::npos);
			// parse answer
			size_t pos = 0, p1;
			int count = 0;
			const std::string tag("(TrFon(");
			while ((p1 = msg.find(tag, pos)) != std::string::npos) {
				p1 += 7; // points to the first "
				int nameStart     = msg.find(',', p1)          +3;
				int nameStop      = msg.find('"', nameStart)   -1;
				int numberStart   = msg.find(',', nameStop)    +3;
				int numberStop    = msg.find('"', numberStart) -1;
				cFonbuchEntry fe;
				fe.number = msg.substr(numberStart, numberStop - numberStart+1);
				fe.name = msg.substr(nameStart, nameStop - nameStart+1);
				convertEntities(fe.name);
				dsyslog("fritzfonbuch.c: (%s / %s)", fe.number.c_str(), fe.name.c_str());
				fonbuchList.push_back(fe);

				pos = p1+10;
				count++;
			}
			dsyslog("fritzfonbuch.c: read %i entries.", count);
			fonbuchRead = true;
		} catch (cTcpException te) {
			dsyslog("fritzfonbuch.c: Exception - %s", te.what());
			sleep(RETRY_DELAY); // delay a possible retry
		}
	} while (!fonbuchRead);
}

std::string &cFritzFonbuch::ResolveToName(std::string &number) {
	for (unsigned int pos=0; pos < fonbuchList.size(); pos++) {
		//cFonbuchEntry fe = fonbuchList[pos];
		if (number.compare(fonbuchList[pos].number) == 0)
			return fonbuchList[pos].name;
	}
	return number;
}

cFonbuchEntry *cFritzFonbuch::RetrieveFonbuchEntry(int id) {
	return &fonbuchList[id];
}

unsigned int cFritzFonbuch::GetFonbuchSize() {
	return fonbuchList.size();
}

bool cFritzFonbuch::InitCall(std::string &number) {
	std::string msg;
	try {
		Login();
		dsyslog("fritzfonbuch.c: sending fonbuch request.");
		cTcpClient tc(fritzboxConfig.url, PORT_WWW);
		tc  <<	"POST /cgi-bin/webcm HTTP/1.1\n"
			<<	"Content-Type: application/x-www-form-urlencoded\n"
			<<	"Content-Length: "
			<<	107 + number.length()
			<<	"\n\n"
			<<	"getpage=..%2Fhtml%2Fde%2Fmenus%2Fmenu2.html&var%3Apagename=fonbuch&var%3Amenu=home&telcfg%3Acommand%2FDial="
			<<	number
			<<	"\n";
		do {
			tc >> msg;
			usleep(100000);
		} while (msg.find(std::string("</html>")) == std::string::npos);

	} catch (cTcpException te) {
		dsyslog("fritzfonbuch.c: Exception - %s", te.what());
		return false;
	}
	return true;	
}

bool cFritzFonbuch::LoadCallList() {
	std::string msg;
	try {
		Login();
		dsyslog("fritzfonbuch.c: sending callList request.");
		cTcpClient tc(fritzboxConfig.url, PORT_WWW);
		tc << "GET /cgi-bin/webcm?getpage=..%2Fhtml%2Fde%2FFRITZ%21Box_Anrufliste.csv HTTP/1.1\n\n\0";
		do {
			tc >> msg;
			usleep(100000);
		} while (msg.find(std::string("\n\n")) == std::string::npos);

	} catch (cTcpException te) {
		dsyslog("fritzfonbuch.c: Exception - %s", te.what());
		return false;
	}
	// parse answer
	callListIn.clear();
	callListOut.clear();
	callListMissed.clear();
	size_t pos = 0, p1;
	int count = 0;
	pos = msg.find(";Dauer", pos);
	while ((p1 = msg.find("\n", pos)) != std::string::npos && msg[p1+1] != '\n') {
		int type          = p1                           +1;
		int dateStart     = msg.find(';', type)          +1;
		int nameStart     = msg.find(';', dateStart)     +1;
		int numberStart   = msg.find(';', nameStart)     +1;
		int lNameStart    = msg.find(';', numberStart)   +1;
		int lNumberStart  = msg.find(';', lNameStart)    +1;
		int durationStart = msg.find(';', lNumberStart)  +1;
		int durationStop  = msg.find("\n", durationStart)-1;
		cCallEntry ce;
		ce.date           = msg.substr(dateStart,     nameStart     - dateStart     -1);
		ce.remoteName     = msg.substr(nameStart,     numberStart   - nameStart     -1);
		ce.remoteNumber   = msg.substr(numberStart,   lNameStart    - numberStart   -1);
		ce.localName      = msg.substr(lNameStart,    lNumberStart  - lNameStart    -1);
		ce.localNumber    = msg.substr(lNumberStart,  durationStart - lNumberStart  -1);
		ce.duration       = msg.substr(durationStart, durationStop -  durationStart +1);
		dsyslog("fritzfonbuch.c: (%s / %s / %s / %s / %s / %s)", ce.date.c_str(), ce.duration.c_str(), ce.remoteNumber.c_str(), ce.remoteName.c_str(), ce.localName.c_str(), ce.localNumber.c_str());
		switch (atoi(msg.substr(type, 1).c_str())) {
		case INCOMING:
			callListIn.push_back(ce);
			break;
		case OUTGOING:
			callListOut.push_back(ce);
			break;
		case MISSED:
			callListMissed.push_back(ce);
			break;
		}
		pos = p1+1;
		count++;
	}
	dsyslog("fritzfonbuch.c: CallList -> read %i entries.", count);	
	return true;
}

cCallEntry *cFritzFonbuch::RetrieveCallListEntry(callType type, int id) {
	switch (type) {
	case INCOMING:
		return &callListIn[id];
	case OUTGOING:
		return &callListOut[id];
	case MISSED:
		return &callListMissed[id];
	default:
		return NULL;
	}
}

unsigned int cFritzFonbuch::GetCallListSize(callType type) {
	switch (type) {
	case INCOMING:
		return callListIn.size();
	case OUTGOING:
		return callListOut.size();
	case MISSED:
		return callListMissed.size();
	default:
		return 0;
	}
}

==> vdr-plugin-fritzbox_0.0.4.orig.tar.gz: fritzbox-0.0.4/fritzfonbuch.c <==
#include "fritzfonbuch.h"
#include "tcpclient.h"
#include "setup.h"
#include <vdr/tools.h>
#include <vdr/config.h>
#include <sstream>

const char *Entities[97][2] = {
	{"&nbsp;",  " "},
	{"&iexcl;", ""},
	{"&cent;",  ""},
	{"&pound;", ""},
	{"&curren;",""},
	{"&yen;",   ""},
	{"&brvbar;",""},
	{"&sect;",  ""},
	{"&uml;",   ""},
	{"&copy;",  ""},
	{"&ordf;",  ""},
	{"&laquo;", ""},
	{"&not;",   ""},
	{"&shy;",   ""},
	{"&reg;",   ""},
	{"&macr;",  ""},
	{"&deg;",   ""},
	{"&plusmn;",""},
	{"&sup2;",  ""},
	{"&sup3;",  ""},
	{"&acute;", ""},
	{"&micro;", ""},
	{"&para;",  ""},
	{"&middot;",""},
	{"&cedil;", ""},
	{"&sup1;",  ""},
	{"&ordm;",  ""},
	{"&raquo;", ""},
	{"&frac14;",""},
	{"&frac12;",""},
	{"&frac34;",""},
	{"&iquest;",""},
	{"&Agrave;",""},
	{"&Aacute;",""},
	{"&Acirc;", ""},
	{"&Atilde;",""},
	{"&Auml;",  ""},
	{"&Aring;", ""},
	{"&AElig;", ""},
	{"&Ccedil;",""},
	{"&Egrave;",""},
	{"&Eacute;",""},
	{"&Ecirc;", ""},
	{"&Euml;",  ""},
	{"&Igrave;",""},
	{"&Iacute;",""},
	{"&Icirc;", ""},
	{"&Iuml;",  ""},
	{"&ETH;",   ""},
	{"&Ntilde;",""},
	{"&Ograve;",""},
	{"&Oacute;",""},
	{"&Ocirc;", ""},
	{"&Otilde;",""},
	{"&Ouml;",  ""},
	{"&times;", ""},
	{"&Oslash;",""},
	{"&Ugrave;",""},
	{"&Uacute;",""},
	{"&Ucirc;", ""},
	{"&Uuml;",  ""},
	{"&Yacute;",""},
	{"&THORN;", ""},
	{"&szlig;", ""},
	{"&agrave;",""},
	{"&aacute;",""},
	{"&acirc;", ""},
	{"&atilde;",""},
	{"&auml;",  ""},
	{"&aring;", ""},
	{"&aelig;", ""},
	{"&ccedil;",""},
	{"&egrave;",""},
	{"&eacute;",""},
	{"&ecirc;", ""},
	{"&euml;",  ""},
	{"&igrave;",""},
	{"&iacute;",""},
	{"&icirc;", ""},
	{"&iuml;",  ""},
	{"&eth;",   ""},
	{"&ntilde;",""},
	{"&ograve;",""},
	{"&oacute;",""},
	{"&ocirc;", ""},
	{"&otilde;",""},
	{"&ouml;",  ""},
	{"&divide;",""},
	{"&oslash;",""},
	{"&ugrave;",""},
	{"&uacute;",""},
	{"&ucirc;", ""},
	{"&uuml;",  ""},
	{"&yacute;",""},
	{"&thorn;", ""},
	{"&yuml;",  ""},
	{"&amp;",   "&"},
};

std::string &convertEntities(std::string &s) {
	if (s.find("&") != std::string::npos) {
		for (int i=0; i<97; i++) {
			std::string::size_type pos = s.find(Entities[i][0]);
			if (pos != std::string::npos) {
				s.replace(pos, strlen(Entities[i][0]), Entities[i][1]);
				i--; //search for the same entity again
			}
		}
#if VDRVERSNUM >= 10503
		// convert the string from latin15 to current system character table
		cCharSetConv *conv = new cCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
		const char *s_converted = conv->Convert(s.c_str());
		s = s_converted;
		delete (conv);
#endif		
	}	
	return s;
}


cFritzFonbuch::cFritzFonbuch() 
:cThread("cFritzFonbuch")
{
	fonbuchRead = false;
	Start();
}

cFritzFonbuch::~cFritzFonbuch() {
}

void cFritzFonbuch::Login() {
	std::ostringstream sMsg;

	dsyslog("fritzfonbuch.c: sending login request to fritz.box.");
	cTcpClient *tc = new cTcpClient(fritzboxConfig.url, PORT_WWW);
	*tc  <<		"POST /cgi-bin/webcm HTTP/1.1\n"
		<<  	"Content-Type: application/x-www-form-urlencoded\n"
		<<  	"Content-Length: "
		<<  	23 + fritzboxConfig.password.size()
		<<  	"\n\nlogin:command/password="
		<<  	fritzboxConfig.password
		<<  	"\n";
	do {
		*tc >> sMsg;
		usleep(100000);
	} while (sMsg.str().find(std::string("</html>")) == std::string::npos);
	delete tc;
}

void cFritzFonbuch::Action() {
	std::string msg;
	do {
		try {
			fonbuchList.clear();
			Login();

			dsyslog("fritzfonbuch.c: sending fonbuch request.");
			cTcpClient tc(fritzboxConfig.url, PORT_WWW);
			tc  <<	"GET /cgi-bin/webcm?getpage=../html/de/menus/menu2.html"
					"&var:lang=de&var:pagename=fonbuch&var:menu=fon HTTP/1.1\n\n\0";

			do {
				tc >> msg;
				usleep(100000);
			} while (msg.find(std::string("</html>")) == std::string::npos);
			// parse answer
			size_t pos = 0, p1;
			int count = 0;
			const std::string tag("(TrFon(");
			while ((p1 = msg.find(tag, pos)) != std::string::npos) {
				p1 += 7; // points to the first "
				int nameStart     = msg.find(',', p1)          +3;
				int nameStop      = msg.find('"', nameStart)   -1;
				int numberStart   = msg.find(',', nameStop)    +3;
				int numberStop    = msg.find('"', numberStart) -1;
				cFonbuchEntry fe;
				fe.number = msg.substr(numberStart, numberStop - numberStart+1);
				fe.name = msg.substr(nameStart, nameStop - nameStart+1);
				convertEntities(fe.name);
				//dsyslog("fritzfonbuch.c: (%s / %s)", fe.number.c_str(), fe.name.c_str());
				fonbuchList.push_back(fe);

				pos = p1+10;
				count++;
			}
			dsyslog("fritzfonbuch.c: read %i entries.", count);
			fonbuchRead = true;
		} catch (cTcpException te) {
			dsyslog("fritzfonbuch.c: Exception - %s", te.what());
			sleep(RETRY_DELAY); // delay a possible retry
		}
	} while (!fonbuchRead);
}

std::string &cFritzFonbuch::ResolveToName(std::string &number) {
	for (unsigned int pos=0; pos < fonbuchList.size(); pos++) {
		//cFonbuchEntry fe = fonbuchList[pos];
		if (number.compare(fonbuchList[pos].number) == 0)
			return fonbuchList[pos].name;
	}
	return number;
}

cFonbuchEntry *cFritzFonbuch::RetrieveFonbuchEntry(int id) {
	return &fonbuchList[id];
}

unsigned int cFritzFonbuch::GetFonbuchSize() {
	return fonbuchList.size();
}

bool cFritzFonbuch::InitCall(std::string &number) {
	std::string msg;
	try {
		Login();
		dsyslog("fritzfonbuch.c: sending fonbuch request.");
		cTcpClient tc(fritzboxConfig.url, PORT_WWW);
		tc  <<	"POST /cgi-bin/webcm HTTP/1.1\n"
			<<	"Content-Type: application/x-www-form-urlencoded\n"
			<<	"Content-Length: "
			<<	107 + number.length()
			<<	"\n\n"
			<<	"getpage=..%2Fhtml%2Fde%2Fmenus%2Fmenu2.html&var%3Apagename=fonbuch&var%3Amenu=home&telcfg%3Acommand%2FDial="
			<<	number
			<<	"\n";
		do {
			tc >> msg;
			usleep(100000);
		} while (msg.find(std::string("</html>")) == std::string::npos);

	} catch (cTcpException te) {
		dsyslog("fritzfonbuch.c: Exception - %s", te.what());
		return false;
	}
	return true;	
}

bool cFritzFonbuch::LoadCallList() {
	std::string msg;
	try {
		Login();
		dsyslog("fritzfonbuch.c: sending callList request.");
		// force an update of the fritz!box csv list
		cTcpClient tc2(fritzboxConfig.url, PORT_WWW);
		tc2 << "GET /cgi-bin/webcm?getpage=../html/de/menus/menu2.html&var:lang=de&var:pagename=foncalls&var:menu=fon HTTP/1.1\n\n\0";
		// retrieve csv list
		cTcpClient tc(fritzboxConfig.url, PORT_WWW);
		tc << "GET /cgi-bin/webcm?getpage=..%2Fhtml%2Fde%2FFRITZ%21Box_Anrufliste.csv HTTP/1.1\n\n\0";
		do {
			tc >> msg;
			usleep(100000);
		} while (msg.find(std::string("\n\n")) == std::string::npos);

	} catch (cTcpException te) {
		dsyslog("fritzfonbuch.c: Exception - %s", te.what());
		return false;
	}
#if VDRVERSNUM >= 10503
	// convert answer to current SystemCodeSet (we assume, Fritz!Box sends its answer in latin15)
	cCharSetConv *conv = new cCharSetConv("ISO-8859-15", cCharSetConv::SystemCharacterTable());
	const char *msg_converted = conv->Convert(msg.c_str());
	msg = msg_converted;
	delete(conv);
#endif	
	// parse answer
	callListIn.clear();
	callListOut.clear();
	callListMissed.clear();
	size_t pos = 0, p1;
	int count = 0;
	pos = msg.find(";Dauer", pos);
	while ((p1 = msg.find("\n", pos)) != std::string::npos && msg[p1+1] != '\n') {
		int type          = p1                           +1;
		int dateStart     = msg.find(';', type)          +1;
		int nameStart     = msg.find(';', dateStart)     +1;
		int numberStart   = msg.find(';', nameStart)     +1;
		int lNameStart    = msg.find(';', numberStart)   +1;
		int lNumberStart  = msg.find(';', lNameStart)    +1;
		int durationStart = msg.find(';', lNumberStart)  +1;
		int durationStop  = msg.find("\n", durationStart)-1;
		cCallEntry ce;
		ce.date           = msg.substr(dateStart,     nameStart     - dateStart     -1);
		ce.remoteName     = msg.substr(nameStart,     numberStart   - nameStart     -1);
		ce.remoteNumber   = msg.substr(numberStart,   lNameStart    - numberStart   -1);
		ce.localName      = msg.substr(lNameStart,    lNumberStart  - lNameStart    -1);
		ce.localNumber    = msg.substr(lNumberStart,  durationStart - lNumberStart  -1);
		ce.duration       = msg.substr(durationStart, durationStop -  durationStart +1);
		//dsyslog("fritzfonbuch.c: (%s / %s / %s / %s / %s / %s)", ce.date.c_str(), ce.duration.c_str(), ce.remoteNumber.c_str(), ce.remoteName.c_str(), ce.localName.c_str(), ce.localNumber.c_str());
		switch (atoi(msg.substr(type, 1).c_str())) {
		case INCOMING:
			callListIn.push_back(ce);
			break;
		case OUTGOING:
			callListOut.push_back(ce);
			break;
		case MISSED:
			callListMissed.push_back(ce);
			break;
		}
		pos = p1+1;
		count++;
	}
	dsyslog("fritzfonbuch.c: CallList -> read %i entries.", count);	
	return true;
}

cCallEntry *cFritzFonbuch::RetrieveCallListEntry(callType type, int id) {
	switch (type) {
	case INCOMING:
		return &callListIn[id];
	case OUTGOING:
		return &callListOut[id];
	case MISSED:
		return &callListMissed[id];
	default:
		return NULL;
	}
}

unsigned int cFritzFonbuch::GetCallListSize(callType type) {
	switch (type) {
	case INCOMING:
		return callListIn.size();
	case OUTGOING:
		return callListOut.size();
	case MISSED:
		return callListMissed.size();
	default:
		return 0;
	}
}
